//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Mas Awoo"
#property link      "mailto:mas_awoo@yahoo.com"
//--------------------------------------

#define MAGICMA_M1  20050610
#define MAGICMA_M2  21173257

extern double MinLots            = 0.1;
extern double MaxLotSize         = 99;
extern bool   AutoSL             = false;
extern double StopLoss           = 299;
extern double TakeProfit1        = 55;
extern double TakeProfit2        = 77;
extern double TakeProfit3        = 111;
extern double TakeProfit4        = 135;
extern double TakeProfit5        = 157;
extern double TakeProfit6        = 222;
extern double TakeProfit7        = 1111;
extern double TrailingStop       = 33;
extern double MaximumRisk        = 0.02;
extern double IncreaseFactor     = 2;
extern double xAtr               = 0.8;
extern double xAtr_High          = 0.9;
extern double xAtr_Low           = 0.69;
extern double MA_PERIODE_M1      = 11;
extern double ATR_PERIODE        = 37;
extern double MA_PERIODE_M2      = 120;
extern double Force_Period       = 77;
extern double K_Period_M1        = 8;
extern double D_Period_M1        = 3;
extern double Slowing_Period_M1  = 3;
extern double K_Period_M2        = 5;
extern double D_Period_M2        = 3;
extern double Slowing_Period_M2  = 3;
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && (OrderMagicNumber()==MAGICMA_M1 || OrderMagicNumber()==MAGICMA_M2) )
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }

//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=MinLots;
   int    orders=OrdersHistoryTotal();     // history orders total
   int    losses=0, profit=0;                  // number of losses orders without a break
//---- select lot size

 lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);

//---- calcuulate number of losses orders without a break
   if(IncreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++; lot=OrderLots();
        }
      if(losses>1) lot=lot*IncreaseFactor;
     }
//---- return lot size
   if(lot<MinLots) lot=MinLots;
   if(lot>MaxLotSize ) lot=MaxLotSize;
   return(lot);
  }
  
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpenM1()
  {
   double AC1, AC2, AC11, AC22, AO1, AO2,St, Sts, Atr, MA, SL;
   int    res;
//---- go trading only for first tiks of new bar
   if (Volume[0]>1)return;
//---- get Moving Average 
   St=iStochastic(NULL,0,K_Period_M1,D_Period_M1,Slowing_Period_M1,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,K_Period_M1,D_Period_M1,Slowing_Period_M1,MODE_SMA,0,MODE_SIGNAL,0);
   AC1=iCustom(NULL, 0, "AC1",1,0);
   AC2=iCustom(NULL, 0, "AC2",2,0);
   AC11=iCustom(NULL, 0, "AC1",1,1);
   AC22=iCustom(NULL, 0, "AC2",2,1);
   Atr=iATR(NULL,0,ATR_PERIODE,0);
   MA=iMA(NULL,0,MA_PERIODE_M1,0,MODE_SMA,0,0);

//---- Buy conditions
   if(AC1>0 && AC11<=0 && St>50 && St>Sts && ((Atr>xAtr&& Atr<xAtr_High)||(Atr<xAtr && Atr<xAtr_Low)) && Open[0]>MA)
     {
      if (AutoSL==true ) SL=Ask-(iATR(NULL,0,ATR_PERIODE,0)*8);
      if (AutoSL==false ) SL=Ask-StopLoss;
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit1*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit2*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit3*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit4*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit5*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit6*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit7*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      return;
     }

  //----
//---- Sell conditions
   if(AC2<0 && AC22>=0 && St<50 && St<Sts && ((Atr>xAtr && Atr>xAtr_High)||(Atr<xAtr && Atr>xAtr_Low))&& Open[0]<MA)
     {
      if (AutoSL==true ) SL=Bid+(iATR(NULL,0,ATR_PERIODE,0)*8);
      if (AutoSL==false ) SL=Bid+StopLoss;
      
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit1*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit2*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit3*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit4*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit5*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit6*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit7*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      return;
     }

  //----
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpenM2()
  {
   double AC1, AC2, AC11, AC22, AO1, AO2,St, Sts, Force, MA, SL;
   int    res;
//---- go trading only for first tiks of new bar
   if (Volume[0]>1)return;
//---- get Moving Average 
   St=iStochastic(NULL,0,K_Period_M2,D_Period_M2,Slowing_Period_M2,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,K_Period_M2,D_Period_M2,Slowing_Period_M2,MODE_SMA,0,MODE_SIGNAL,0);
   AC1=iCustom(NULL, 0, "AC1",1,0);
   AC2=iCustom(NULL, 0, "AC2",2,0);
   AC11=iCustom(NULL, 0, "AC1",1,1);
   AC22=iCustom(NULL, 0, "AC2",2,1);
   Force=iForce(NULL, 0, Force_Period,MODE_SMA,PRICE_CLOSE,1);
   MA=iMA(NULL,0,MA_PERIODE_M2,0,MODE_SMA,0,0);
   

//---- Buy conditions
   if(AC1>0 && AC11<0 && St>80 && St>Sts && Force>0 && iOpen(NULL,0,0)>MA)
     {      
      if (AutoSL==true ) SL=Ask-(iATR(NULL,0,ATR_PERIODE,0)*8);
      if (AutoSL==false ) SL=Ask-StopLoss;

      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit1*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit2*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit3*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit4*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit5*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit6*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,SL,Ask+TakeProfit7*Point,"Mas Awoo",MAGICMA_M1,0,Blue);
      return;
     }

  //----
//---- Sell conditions
   if(AC2<0 && AC22>0 && St<20 && St<Sts && Force<0 && iOpen(NULL,0,0)<MA)
     {
      if (AutoSL==true ) SL=Bid+(iATR(NULL,0,ATR_PERIODE,0)*8);
      if (AutoSL==false ) SL=Bid+StopLoss;
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit1*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit2*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit3*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit4*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit5*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit6*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,SL,Bid-TakeProfit7*Point,"Mas Awoo",MAGICMA_M1,0,Red);
      return;
     }

  //----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double AC1, AC2, AC11, AC22, AO1, AO2,St, Sts, Atr, MA;
   int    res;
//---- go trading only for first tiks of new bar
//---- get Moving Average 
   St=iStochastic(NULL,0,K_Period_M1,D_Period_M1,Slowing_Period_M1,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,K_Period_M1,D_Period_M1,Slowing_Period_M1,MODE_SMA,0,MODE_SIGNAL,0);
   AC1=iCustom(NULL, 0, "AC1",1,0);
   AC2=iCustom(NULL, 0, "AC2",2,0);
   AC11=iCustom(NULL, 0, "AC1",1,1);
   AC22=iCustom(NULL, 0, "AC2",2,1);
   Atr=iATR(NULL,0,ATR_PERIODE,0);
   MA=iMA(NULL,0,MA_PERIODE_M1,0,MODE_SMA,0,0);

//---- get Moving Average 
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA_M1 || OrderMagicNumber()!=MAGICMA_M2 || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(AC2<0 && AC22>=0 && St<25 && St<Sts && ((Atr>xAtr && Atr>xAtr_High)||(Atr<xAtr && Atr>xAtr_Low))&& Open[0]<MA)
          OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(AC1>0 && AC11<=0 &&  St>75 && St>Sts && ((Atr>xAtr&& Atr<xAtr_High)||(Atr<xAtr && Atr<xAtr_Low)) && Open[0]>MA)
         OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }

  }
//+------------------------------------------------------------------+
//| Check For TrailingStop                                           |
//+------------------------------------------------------------------+
void TrailingStop ()
{
double total, cnt;
   total=OrdersTotal();
   for(cnt=0;cnt<total;cnt++)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL && 
         OrderSymbol()==Symbol())   
        {
         if(OrderType()==OP_BUY)  
           {
            if(TrailingStop>0)  
              {                 
               if(Bid-OrderOpenPrice()>Point*TrailingStop)
                 {
                  if(OrderStopLoss()<Bid-Point*TrailingStop)
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Blue);
                     return(0);
                    }
                 }
              }
           }
         else //
           {
            if(TrailingStop>0)  
              {                 
               if((OrderOpenPrice()-Ask)>(Point*TrailingStop))
                 {
                  if(OrderStopLoss()==0.0 || 
                     OrderStopLoss()>(Ask+Point*TrailingStop))
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red);
                     return(0);
                    }
                 }
              }
           }
        }
     }
     return(0);
     }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForElseOpen()
  {
//---- go trading only for first tiks of new bar

//---- get Moving Average 
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderSymbol()==Symbol())   
      {
      if(OrderMagicNumber()!=MAGICMA_M1 ) CheckForOpenM1();break;
      if(OrderMagicNumber()!=MAGICMA_M2 ) CheckForOpenM2();break;
      }
      //---- check order type 
      }
}
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) 
   CheckForOpenM1();
   CheckForOpenM2();
   if (CalculateCurrentOrders(Symbol())>0) 
   CheckForElseOpen();
   CheckForClose();
   TrailingStop ();
   
//----
  }
//+------------------------------------------------------------------+